use std::fs;
use mailparse::parse_mail;
use super::util::{parse_email, xml_safe as x};
use crate::models::*;
use scraper::{Html as scraper_html, Selector};

fn feed(
    feed_title: &str,
    feed_link: &str,
    updated: &str,
    author_name: &str,
    author_email: &str,
    entry_list: &str,
) -> String {
    format!(
        r#"<?xml version="1.0" encoding="utf-8"?>
        <feed xmlns="http://www.w3.org/2005/Atom">
        <title>{feed_title}</title>
        <link href="{feed_link}"/>
        <updated>{updated}</updated>
        <author>
        <name>{author_name}</name>
        <email>{author_email}</email>
        </author>
        <id>{feed_link}</id>
        {entry_list}
        </feed>"#,
    )
}

fn message(
    title: &str,
    item_link: &str,
    entry_id: &str,
    updated: &str,
    author_name: &str,
    author_email: &str,
    content: &str,
) -> String {
    format!(
        r#"<entry>
        <title>{title}</title>
        <link href="{item_link}"/>
        <id>{entry_id}</id>
        <updated>{updated}</updated>
        <author>
        <name>{author_name}</name>
        <email>{author_email}</email>
        </author>
        <content type="text/plain">
        {content}
        </content>
        </entry>
        "#,
    )
}

impl StrMessage {
    pub fn to_xml(&self) -> String {
        let msg = self;

        let data = match fs::read_to_string(self.original_path.clone()) {
            Ok(content) => content,
            Err(e) => {
                eprintln!("Error reading file: {}", e);
                return "Error loading content".to_string();
            }
        };
        
        let parsed_mail = match parse_mail(&data.as_bytes()) {
            Ok(mail) => mail,
            Err(e) => return format!("Error parsing email: {}", e),
        };
        
        let html = parse_email(&parsed_mail).unwrap_or_else(|_| msg.body.clone());
        
        let document = scraper_html::parse_document(&*html);
        let body = Selector::parse("body").unwrap();
        
        // Traverse the parsed HTML and extract all text from the body
        let text = document.select(&body)
            .next().unwrap()
            .text()
            .collect::<Vec<_>>()
            .join(" ");
        
        
        // let body = match self.flowed {
        //     true => unformat_flowed(&self.body),
        //     false => self.body.clone(),
        // };
        message(
            &x(&msg.subject),
            &x(&self.url),
            &x(&msg.id),
            &msg.received.to_rfc3339(),
            &x(&msg.from.clone().name.unwrap_or(msg.from.clone().address)),
            &x(&msg.from.address),
            &x(&text),
        )
    }
}

impl Thread {
    pub fn to_xml(&self) -> String {
        let mut entry_list = String::new();
        for msg in &self.messages {
            entry_list.push_str(&msg.to_xml());
        }
        // Sometimes its unclear whether to do stuff like this in models.rs or here. could refactor
        let root = &self.messages[0];
        feed(
            &root.subject,
            &self.url,
            &root.received.to_rfc3339(),
            root.from.name.as_ref().unwrap_or(&root.from.address),
            &root.from.address,
            &entry_list,
        )
    }
}
